@@ -15,7 +15,7 @@ import ctypes as ct
 import numpy as np
 from numpy.random import MT19937, Generator
 from scipy.optimize import OptimizeResult
-from fcmaes.cmaes import _check_bounds
+from fcmaes.cmaes import _check_bounds, serial, parallel
 
 os.environ['MKL_DEBUG_CPU_TYPE'] = '5'
 
@@ -30,7 +30,8 @@ def minimize(fun,
              stop_fittness = None, 
              is_terminate = None, 
              rg = Generator(MT19937()),
-             runid=0):   
+             runid=0,
+             workers = None):   
     """Minimization of a scalar function of one or more variables using a 
     C++ CMA-ES implementation called via ctypes.
      
@@ -69,7 +70,10 @@ def minimize(fun,
         Random generator for creating random guesses.
     runid : int, optional
         id used by the is_terminate callback to identify the CMA-ES run. 
-            
+    workers : int or None, optional
+        If not workers is None, function evaluation is performed in parallel for the whole population. 
+        Useful for costly objective functions but is deactivated for parallel retry.      
+           
     Returns
     -------
     res : scipy.OptimizeResult
@@ -96,11 +100,12 @@ def minimize(fun,
         use_terminate = False 
     else:
         use_terminate = True 
-    array_type = ct.c_double * n   
-    c_callback = call_back_type(callback(fun))
+    parfun = None if workers is None else parallel(fun, workers)
+    array_type = ct.c_double * n 
+    c_callback_par = call_back_par(callback_par(fun, parfun))
     c_is_terminate = is_terminate_type(is_terminate)
     try:
-        res = optimizeACMA_C(runid, c_callback, n, array_type(*guess), array_type(*lower), array_type(*upper), 
+        res = optimizeACMA_C(runid, c_callback_par, n, array_type(*guess), array_type(*lower), array_type(*upper), 
                            array_type(*input_sigma), max_iterations, max_evaluations, stop_fittness, mu, 
                            popsize, accuracy, use_terminate, c_is_terminate, int(rg.uniform(0, 2**32 - 1)))
 
@@ -110,8 +115,12 @@ def minimize(fun,
         iterations = int(res[n+2])
         stop = int(res[n+3])
         freemem(res)
+        if not parfun is None:
+            parfun.stop() # stop all parallel evaluation processes
         return OptimizeResult(x=x, fun=val, nfev=evals, nit=iterations, status=stop, success=True)
     except Exception as ex:
+        if not workers is None:
+            fun.stop() # stop all parallel evaluation processes
         return OptimizeResult(x=None, fun=sys.float_info.max, nfev=0, nit=0, status=-1, success=False)
 
 class callback(object):
@@ -126,6 +135,36 @@ class callback(object):
         except Exception:
             return sys.float_info.max
 
+#https://stackoverflow.com/questions/7543675/how-to-convert-pointer-to-c-array-to-python-array
+
+class callback_par(object):
+    
+    def __init__(self, fun, parfun):
+        self.fun = fun
+        self.parfun = parfun
+    
+    def __call__(self, popsize, n, xs_, ys_):
+        try:
+            #xall = np.array(np.fromiter(xs_, dtype = np.float64, count = popsize*n))
+            
+            arrType = ct.c_double*(popsize*n)
+            addr = ct.addressof(xs_.contents)
+            xall = np.frombuffer(arrType.from_address(addr))
+            
+            if self.parfun is None:
+                for p in range(popsize):
+                    ys_[p] = self.fun(xall[p*n : (p+1)*n])
+            else:    
+                xs = []
+                for p in range(popsize):
+                    x = xall[p*n : (p+1)*n]
+                    xs.append(x)
+                ys = self.parfun(xs)
+                for p in range(popsize):
+                    ys_[p] = ys[p]
+        except Exception as ex:
+            print (ex)
+
 def _is_terminate_false(runid, iterations, val):
     return False 
   
@@ -138,11 +177,12 @@ elif 'mac' in sys.platform:
 else:
     libcmalib = ct.cdll.LoadLibrary(basepath + '/lib/libacmalib.dll')  
 
-call_back_type = ct.CFUNCTYPE(ct.c_double, ct.c_int, ct.POINTER(ct.c_double))  
+call_back_par = ct.CFUNCTYPE(None, ct.c_int, ct.c_int, \
+                                  ct.POINTER(ct.c_double), ct.POINTER(ct.c_double))  
 is_terminate_type = ct.CFUNCTYPE(ct.c_bool, ct.c_long, ct.c_int, ct.c_double)    
 
 optimizeACMA_C = libcmalib.optimizeACMA_C
-optimizeACMA_C.argtypes = [ct.c_long, call_back_type, ct.c_int, \
+optimizeACMA_C.argtypes = [ct.c_long, call_back_par, ct.c_int, \
             ct.POINTER(ct.c_double), ct.POINTER(ct.c_double), ct.POINTER(ct.c_double), \
             ct.POINTER(ct.c_double), ct.c_int, ct.c_int, ct.c_double, ct.c_int, ct.c_int, \
             ct.c_double, ct.c_bool, is_terminate_type, ct.c_long]
