@@ -31,7 +31,7 @@ def minimize(fun,
              popsize = None, 
              max_evaluations = 100000, 
              stop_fittness = None, 
-             keep = 30.0,
+             keep = 200,
              f = 0.5,
              cr = 0.9,
              rg = Generator(MT19937()),
